from MainHandler import Handler
from runCalculations import RunCalculations
import pandas
from gi.repository import Gtk
import os
import gi

gi.require_version('Gtk', '3.0')


class FilterHandler(Handler):
    def __init__(self, builder, handler: Handler):
        super().__init__(builder)
        self.builder = builder
        self.handler = handler
        self.config_varcut_window = self.builder.get_object(
            'config_varcut_window')
        self.config_varcut_window.connect(
            'delete-event', lambda w, e: w.hide() or True)
        self.config_varcut_window = builder.get_object('config_varcut_window')
        self.config_corrcut_window = builder.get_object(
            'config_corrcut_window')
        self.config_autocorrcut_window = builder.get_object(
            'config_autocorrcut_window')

    def run_corrfilter(self, prefix, value) -> bool:
        runMethod = {
            'corrcut': RunCalculations.runCorrCut,
            'autocorrcut': RunCalculations.runAutoCorrCut
        }
        window = {
            'corrcut': self.config_corrcut_window,
            'autocorrcut': self.config_autocorrcut_window
        }
        if prefix not in runMethod:
            return False

        """ TODO:
            In the future, the user will be able to cut the matrix without
            saving it, leaving it temporarily available within the program to
            perform another calculation in the sequence. """
        save = True  # self.builder.get_object(f'{prefix}_save').get_active()
        output = self.builder.get_object(
            f'{prefix}_output').get_text() if save else ""
        new_matrix = runMethod[prefix](
            self.handler.get_X_matrix(), self.handler.get_y_vector(), value, save, output)
        print(
            f'x_matrix:{self.handler.get_X_matrix()}\n\ny_vector:{self.handler.get_y_vector()}\n\nreturn:{new_matrix}')
        # TODO: let user choose whether to replace the active matrix with the new one
        if os.path.isfile(new_matrix):
            self.handler.set_X_matrix(new_matrix)
            self.handler.draw_matrices('matrix')
        window[prefix].hide()
        return True

    def on_varcut_run_button_clicked(self, _) -> None:
        """ Handle Run button from Variance cut screen """
        if self.handler.files_ok():
            value = float(self.builder.get_object('varcut_varcut').get_value())
            # TODO
            """ In the future, the user will be able to cut the matrix without
                saving it, leaving it temporarily available within the program to
                perform another calculation in the sequence. """
            save = True  # self.builder.get_object('varcut_save').get_active()
            output = self.builder.get_object(
                'varcut_output').get_text() if save else ""
            new_matrix = RunCalculations.runVarCut(
                self.handler.get_X_matrix(), value, save, output)
            # TODO: let user choose whether to replace the active matrix with the new one
            if os.path.isfile(new_matrix):
                self.handler.set_X_matrix(new_matrix)
                self.handler.draw_matrices('matrix')
            self.config_varcut_window.hide()

    def on_corrcut_run_button_clicked(self, _) -> None:
        """ Handle Run button from Correlation cut screen """
        if self.handler.files_ok():
            value = float(self.builder.get_object(
                'corrcut_corrcut').get_value())
            self.run_corrfilter('corrcut', value)

    def on_autocorrcut_run_button_clicked(self, _) -> None:
        """ Handle Run button from Autocorrelation cut screen """
        if self.handler.files_ok():
            value = float(self.builder.get_object(
                'autocorrcut_autocorrcut').get_value())
            self.run_corrfilter('autocorrcut', value)

    def on_varcut_save_toggled(self, this) -> None:
        self.on_save_toggled(this, 'varcut')

    def on_corrcut_save_toggled(self, this) -> None:
        self.on_save_toggled(this, 'corrcut')

    def on_autocorrcut_save_toggled(self, this) -> None:
        self.on_save_toggled(this, 'autocorrcut')

    def on_save_toggled(self, this, prefix) -> None:
        """ Handle the toggle of Variance Cut screen save Option """
        box = self.builder.get_object(f'{prefix}_filename_box')
        if this.get_active():
            box.show()
        else:
            box.hide()



import os
from typing import Type, Union, List
import pandas
import qsarmodelingpy.Utils
import Utils
import logging
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from Constants import DEBUG_MODE

import warnings
warnings.filterwarnings("ignore")

class Handler(object):

    __childrens: List[object] = []

    def __init__(self, builder):
        self.builder = builder
        # Saving windows
        self.main_window = builder.get_object('main_window')
        self.results_window = builder.get_object('results_window')
        self.config_OPS_window = builder.get_object('config_OPS_window')
        self.config_GA_window = builder.get_object('config_GA_window')
        self.about_window = builder.get_object('about_window')
        self.csv_file_filter = builder.get_object('open_filter')
        self.main_window_stack = builder.get_object('main_window_stack')
        self.config_varcut_window = builder.get_object('config_varcut_window')
        self.config_corrcut_window = builder.get_object(
            'config_corrcut_window')
        self.config_autocorrcut_window = builder.get_object(
            'config_autocorrcut_window')
        self.config_cv_window = builder.get_object('config_cv_window')
        self.config_yrlno_window = builder.get_object('config_yrlno_window')
        self.config_extval_window = builder.get_object('config_extval_window')

        # Saving elements
        self.main_window_pages = [builder.get_object(
            'main_window_welcome'), builder.get_object('main_window_tables')]
        self.treeview_X = builder.get_object('treeview_X')
        self.treeview_y = builder.get_object('treeview_y')

        # connect destroy signal
        self.main_window.connect('destroy', Gtk.main_quit)
        self.about_window.connect(
            'delete-event', lambda w, e: w.hide() or True)
        self.results_window.connect(
            'delete-event', lambda w, e: w.hide() or True)

        # Setting file filters
        self.csv_file_filter.set_name('CSV Files (*.csv)')

        # Handling files
        self.X_matrix = None
        self.y_vector = None
        self.last_opened_path = ""
        self.last_saved_path = ""

        # TODO: remove these stuff

    def register_handler(self, handler) -> None:
        """Register a new handler 

        Args:
            handler (Handler): The handler to be registered
        """        
        if handler not in self.__childrens:
            self.__childrens.append(handler)
    
    def get_handlers(self) -> list:
        """ Get all handlers """
        return self.__childrens

    def get_handler(self, _type: Type) -> object:
        for handler in self.__childrens:
            if isinstance(handler, _type):
                return handler
        raise ValueError(f"No handler of type {_type} found.")

    def on_menu_openlog_activate(self, _) -> None:
        """ Open the log """
        Utils.open_external(Utils.get_log_file())

    def on_menu_ops_model_activate(self, _) -> None:
        """ Handle menu Generate > OPS """
        self.config_OPS_window.show_all()

    def on_menu_ga_model_activate(self, _) -> None:
        """ Handle menu Generate > GA """
        self.config_GA_window.show_all()

    def on_menu_varcut_activate(self, _) -> None:
        """ Handle menu Filter > Variance cut """
        self.config_varcut_window.show()

    def on_menu_corrcut_activate(self, _) -> None:
        """ Handle menu Filter > Correlation cut """
        self.config_corrcut_window.show()

    def on_menu_autocorrcut_activate(self, _) -> None:
        """ Handle menu Filter > Autocorrelation cut """
        self.config_autocorrcut_window.show()

    def on_menu_cv_activate(self, _) -> None:
        """ Handle menu Validation > Cross Validation """
        self.config_cv_window.show()

    def on_menu_yrlno_activate(self, _) -> None:
        """ Handle menu Validation > Y-Randomization / Leave-N-Out """
        self.config_yrlno_window.show()

    def on_menu_extval_activate(self, _) -> None:
        """ Handle menu Validation > External Validation """
        self.config_extval_window.show()

    def open_file(self, use_last_path=True) -> str:
        """
        Show an Open File dialog

        :param use_last_path: Whether or not to start at the last folder opened
        :return: the selected filename
        """
        file_chooser = Gtk.FileChooserDialog(
            title="Open...", action=Gtk.FileChooserAction.OPEN)
        file_chooser.add_buttons(
            "Cancel", Gtk.ResponseType.CANCEL, "Open", Gtk.ResponseType.OK)
        file_chooser.set_default_response(Gtk.ResponseType.OK)
        file_chooser.add_filter(self.csv_file_filter)
        if self.last_opened_path and use_last_path:
            file_chooser.set_current_folder(self.last_opened_path)
        response = file_chooser.run()
        filename = ""
        if response == Gtk.ResponseType.OK:
            filename = file_chooser.get_filename()
            self.last_opened_path = os.path.dirname(os.path.abspath(filename))
        file_chooser.destroy()
        return filename

    def save_file(self) -> str:
        file_chooser = Gtk.FileChooserDialog(
            title="Save...", action=Gtk.FileChooserAction.SAVE)
        file_chooser.add_buttons(
            "Cancel", Gtk.ResponseType.CANCEL, "Save", Gtk.ResponseType.OK)
        file_chooser.set_default_response(Gtk.ResponseType.OK)
        file_chooser.add_filter(self.csv_file_filter)
        if self.last_saved_path:
            file_chooser.set_current_folder(self.last_saved_path)
        elif self.last_opened_path:
            file_chooser.set_current_folder(self.last_opened_path)
        filename = ""
        response = file_chooser.run()
        if response == Gtk.ResponseType.OK:
            filename = file_chooser.get_filename()
            self.last_saved_path = filename
        file_chooser.destroy()
        return filename

    def on_save_file(self, entry: Gtk.Entry) -> None:
        filename = self.save_file()
        if filename:
            entry.set_text(filename)

    def on_open_file(self, entry: Gtk.Entry) -> None:
        filename = self.open_file()
        if filename:
            entry.set_text(filename)

    def block_menus_until_file_load(self) -> None:
        """ Block menus while files is not properly loaded """
        menus = [
            self.builder.get_object('menu_generate'),
            self.builder.get_object('menu_validate'),
            self.builder.get_object('menu_filter'),
        ]
        if self.X_matrix and self.y_vector:
            for elem in menus:
                elem.set_sensitive(True)
        else:
            for elem in menus:
                elem.set_sensitive(False)

    def on_menu_open_matrix_activate(self, _) -> None:
        """ Handle menu File > Open... > Open matrix """
        filename = self.open_file()
        if filename:
            # self.X_matrix = filename
            logging.debug(f"Selected {filename}.")
            self.set_X_matrix(filename)
            logging.debug(f"Setted {self.get_X_matrix()}.")
            self.draw_matrices('matrix')

    def on_menu_open_vector_activate(self, _) -> None:
        """ Handle menu File > Open... > Open vector """
        filename = self.open_file()
        if filename:
            # self.y_vector = filename
            logging.debug(f"Selected {filename}.")
            self.set_y_vector(filename)
            logging.debug(f"Setted {self.get_y_vector()}.")
            self.draw_matrices('vector')

    def draw_matrices(self, what_to_draw) -> None:
        """ Draw a pandas matrix or vector in the main screen.
            Works like a Factory to draw_pandas_matrix() and draw_pandas_vector(). """
        show = False
        if what_to_draw == 'matrix' and self.X_matrix and os.path.isfile(self.X_matrix):
            # Draw matrix
            self.draw_pandas_matrix(self.treeview_X, self.X_matrix)
            show = True
        if what_to_draw == 'vector' and self.y_vector and os.path.isfile(self.y_vector):
            # Draw vector
            self.draw_pandas_vector(self.treeview_y, self.y_vector)
            show = True

        if show:
            self.main_window_stack.set_visible_child(self.main_window_pages[1])
        else:
            self.main_window_stack.set_visible_child(self.main_window_pages[0])

        self.block_menus_until_file_load()

    def draw_pandas_matrix(self, treeview, path, print_index=True) -> None:
        """ Draws in treeview a pandas matrix from path (csv) """
        df = qsarmodelingpy.Utils.load_matrix(path, usecols=list(range(10)))

        print_et_cetera_column = False
        if df.shape[1] > 10:
            df = df.iloc[:, 0:10]
            print_et_cetera_column = True
        liststore_args = [str] if print_index else []
        liststore_args += [float] * int(df.shape[1]) # type: ignore
        if print_et_cetera_column:
            liststore_args += [str]
        liststore = Gtk.ListStore(*liststore_args)
        df_indexes = df.index.values
        for i in range(df.shape[0]):
            appendix = [str(df_indexes[i])] if print_index else []
            appendix += list(df.iloc[i, :])
            if print_et_cetera_column:
                appendix += ["..."]
            liststore.append(appendix)
        self.clear_treeview(treeview)
        current_model = treeview.get_model()
        if current_model is not None:
            current_model.clear()
        treeview.set_model(liststore)

        # Draw index column
        if print_index:
            renderer_text = Gtk.CellRendererText()
            column_text = Gtk.TreeViewColumn('Molecule', renderer_text, text=0)
            treeview.append_column(column_text)

        # Draw data columns
        for i in range(df.shape[1]):
            renderer_text = Gtk.CellRendererText()
            text = i + 1 if print_index else i
            column_text = Gtk.TreeViewColumn(
                df.columns[i], renderer_text, text=text)
            treeview.append_column(column_text)

        # Draw et cetera column
        if print_et_cetera_column:
            renderer_text = Gtk.CellRendererText()
            text = df.shape[1] + 1 if print_index else df.shape[1]
            column_text = Gtk.TreeViewColumn(
                '...', renderer_text, text=df.shape[1] + 1)
            treeview.append_column(column_text)

    def draw_pandas_vector(self, treeview, path: str) -> None:
        """ Draws in treeview a pandas vector from path (csv/txt) """
        df = pandas.read_csv(path, header=None, sep=",")

        self.clear_treeview(treeview)

        liststore = Gtk.ListStore(float)

        # df should be a vector
        data = list(df.iloc[0, :]) if df.shape[0] == 1 else list(df.iloc[:, 0])

        for i in range(max(df.shape[0], df.shape[1])):
            liststore.append([data[i]])
        treeview.set_model(liststore)

        # Draw data column
        renderer_text = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn('Vector', renderer_text, text=0)
        treeview.append_column(column_text)

    def on_menu_about_activate(self, _) -> None:
        """ Handle menu About """

        self.about_window.run()

    def get_X_matrix(self) -> str:
        return self.X_matrix

    def get_y_vector(self) -> str:
        return self.y_vector

    def set_X_matrix(self, new_value: str) -> None:
        if new_value is None or os.path.isfile(str(new_value)):
            self.X_matrix = new_value
            self.block_menus_until_file_load()
        else:
            raise TypeError(f"File {new_value} is not a valid file.")

    def set_y_vector(self, new_value: str) -> None:
        if new_value is None or os.path.isfile(str(new_value)):
            self.y_vector = new_value
            self.block_menus_until_file_load()
        else:
            raise TypeError(f"File {new_value} is not a valid file.")

    @staticmethod
    def on_auto_state_set(obj, active: bool) -> None:
        """Set an object as active (editable) or not. Usually called by switchers."""
        if active:
            obj.set_value(0)
            obj.set_editable(False)
            obj.set_sensitive(False)
        else:
            obj.set_editable(True)
            obj.set_sensitive(True)

    def files_ok(self) -> bool:
        matp = self.get_X_matrix()
        logging.debug(f"mat = {matp}")
        mat = os.path.isfile(matp)
        logging.debug(f"mat = {mat}")
        vecp = self.get_y_vector()
        logging.debug(f"vec = {vecp}")
        vec = os.path.isfile(vecp)
        logging.debug(f"vec = {vec}")
        return os.path.isfile(self.get_X_matrix()) and os.path.isfile(self.get_y_vector())

    @staticmethod
    def on_close_modal(modal) -> None:
        modal.hide()

    @staticmethod
    def clear_treeview(treeview) -> None:
        columns = treeview.get_columns()
        for col in columns:
            treeview.remove_column(col)

    @staticmethod
    def on_about_window_destroy(_) -> bool:
        return True

    @staticmethod
    def gtk_main_quit(_) -> None:
        Gtk.main_quit()



import logging
import time
import os
import gi
from multiprocessing import Process
from threading import Thread
from Interfaces import ConfigOPSInterface
from MainHandler import Handler
from runCalculations import RunCalculations
from Utils import get_current_time_as_string, set_output_matrix_as_input
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class OPSHandler(Handler):
    def __init__(self, builder, handler: Handler):
        super().__init__(builder)
        self.builder = builder
        self.handler = handler
        self.config_OPS_window = builder.get_object('config_OPS_window')
        self.config_OPS_window.connect(
            'delete-event', lambda w, e: w.hide() or True)
        self.ops_config: ConfigOPSInterface
        self.running_process = None

    def _is_running(self) -> bool:
        if self.running_process is None:
            return False
        if self.running_process.is_alive():
            return True
        else:
            self.running_process.terminate()
            return False

    def OPS_confirm_close(self, dialog, response) -> None:
        dialog.hide()
        if response == Gtk.ResponseType.OK and self._is_running():
            self.on_OPS_cancel_button_clicked(None, force=True)

    def on_OPS_cancel_button_clicked(self, _, force: bool = False) -> None:
        """ Handle OPS cancel button """
        if self._is_running():
            if not force:
                self.builder.get_object('OPS_will_kill_all_window').show()
                return
            else:
                logging.warning("Terminating OPS Calculation.")
                self.running_process.terminate()
                self.running_process = None
        self.config_OPS_window.hide()

    def _keep_updating_progress_bar(self) -> None:
        pbar = self.builder.get_object("OPS_progress_bar")
        pbar.show()
        while self._is_running():
            time.sleep(0.1)
            pbar.pulse()
        else:
            pbar.set_fraction(0)
            # If everything is ok, current matrix will be the filtered one.
            set_output_matrix_as_input(self, self.ops_config)

    def on_OPS_run_button_clicked(self, _) -> None:
        if self.handler.files_ok():
            self.ops_config = {
                'XMatrix': self.handler.get_X_matrix(),
                'yvector': self.handler.get_y_vector(),
                'output_matrix': self.builder.get_object('OPS_output_matrix').get_text(),
                'output_cv': self.builder.get_object('OPS_output_cv').get_text(),
                'output_models': self.builder.get_object('OPS_output_models').get_text(),
                'output_PLS_model': self.builder.get_object('OPS_output_model').get_text(),
                'varcut': self.builder.get_object('ops_varcut').get_value(),
                'corrcut': self.builder.get_object('ops_corrcut').get_value(),
                'autoscale': self.builder.get_object('ops_autoscale').get_active(),
                'lj_transform': self.builder.get_object('ops_ljtransform').get_active(),
                'autocorrcut': self.builder.get_object('ops_autocorrcut').get_value(),
                'latent_vars_ops': self.builder.get_object('ops_latent_vars_OPS').get_value(),
                'latent_vars_model': self.builder.get_object('ops_latent_vars_model').get_value(),
                'ops_window': self.builder.get_object('ops_OPS_window').get_value(),
                'ops_increment': self.builder.get_object('ops_OPS_increment').get_value(),
                'vars_percentage': self.builder.get_object('ops_vars_percentage').get_value(),
                'models_to_save': self.builder.get_object('ops_models_to_save').get_value(),
                'yrand': self.builder.get_object('ops_yrand').get_value(),
                'lno': self.builder.get_object('ops_lno').get_value(),
                'ops_type': 'f' if self.builder.get_object('ops_feed_ops').get_active() else 's'
            }

            date_time = get_current_time_as_string()
            if not self.ops_config['output_matrix']:
                self.ops_config['output_matrix'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()), "OPS_output_matrix_{}.csv".format(date_time))
            if not self.ops_config['output_cv']:
                self.ops_config['output_cv'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()), "OPS_output_CV_{}.csv".format(date_time))
            if not self.ops_config['output_models']:
                self.ops_config['output_models'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()), "OPS_output_models_{}.json".format(date_time))
            if not self.ops_config['output_PLS_model']:
                self.ops_config['output_PLS_model'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()), "OPS_output_model_{}.csv".format(date_time))

            self.running_process = Process(target=OPSHandler.call_runner, args=(self.ops_config,))
            self.running_process.start()
            pbthread = Thread(target=self._keep_updating_progress_bar)
            pbthread.start()
            logging.debug(f"Started PID = {self.running_process.pid}")
        else:
            print("Please, go to File > Open... before run a calculation.")

    @staticmethod
    def call_runner(config: ConfigOPSInterface) -> None:
        logging.debug("Calling OPS runner.")
        RunCalculations.runOPS(config)
        logging.debug("Calculation done.")

    


from qsarmodelingpy.cross_validation_class import CrossValidation
import os
from typing import Dict, Callable, Optional
try:
    from typing import TypedDict # Python 3.8+
except ImportError:
    from typing_extensions import TypedDict # Python 3.7-
import logging

import pandas as pd
from MainHandler import Handler
import gi

from Plots import CrossValidationPlots, Plots
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.overrides import Gtk as Gtk3

class ResultWindowTexts(TypedDict):
    toptext: Optional[str]
    title: Optional[str]
    statistics: Optional[str]

class ResultsHandler(Handler):
    def __init__(self, builder):
        super().__init__(builder)
        self.builder = builder
        self.window = self.builder.get_object('results_window')
        self.window.connect(
            'delete-event', lambda w, e: w.hide() or True)
        # self.window.show_all()

        self.plot_type_selector: Gtk3.ComboBox = self.builder.get_object(
            'plot_selection_combo')

        # TODO: remove this
        # self.show(self.get_object_for_testing_TODO(), CrossValidationPlots())

    def show(self, dataobject, plotter: Plots, texts: Optional[ResultWindowTexts]=None):
        if texts is None:
            texts = ResultWindowTexts(toptext=None, title=None, statistics=None)

        self.dataobject = dataobject
        self.plotter = plotter
        self.init_selector(self.plotter.get_methods())
        self.set_texts(**texts)
        self.window.show_all()

    def set_texts(self, toptext: Optional[str] = "Your job is done.", statistics: Optional[str] = "", title: Optional[str] = ""):
        self.builder.get_object('results_statistics_toptext').set_text(toptext)

        # Set statistics text
        buffer = self.builder.get_object('results_statistics_textview').get_buffer()
        buffer.set_text(statistics)

        # Set window title
        if title != "": 
            self.window.set_title(title)

    def init_selector(self, methods: Dict[str, Callable]):
        model = Gtk.ListStore(str)
        for key in methods:
            model.append([key])
        self.plot_type_selector.set_model(model)
        self.plot_type_selector.set_entry_text_column(0)
        renderer_text = Gtk.CellRendererText()
        self.plot_type_selector.clear()
        self.plot_type_selector.pack_start(renderer_text, False)
        self.plot_type_selector.add_attribute(renderer_text, "text", 0)
        self.plot_type_selector.set_active(0)

    @staticmethod
    def get_selected_method(elem: Gtk3.ComboBox) -> str:
        tree_iter = elem.get_active_iter()
        if tree_iter is not None:
            model = elem.get_model()
            name = model[tree_iter][0]
            return name
        else:
            entry = elem.get_child()
            return entry.get_text()

    def on_plot(self, combobox: Gtk.ComboBox) -> None:
        self.plotter.get_methods()[self.get_selected_method(
            combobox)](self.dataobject)

    # TODO: remove this
    @staticmethod
    def get_object_for_testing_TODO() -> CrossValidation:
        """This is for TESTING ONLY and should be removed

        Returns:
            CrossValidation: The objetct to plot
        """
        directory = "/home/helitonmrf/Documents/QSAR/cancer_prostata/resultados"
        X_matrix_file = "filtered_10_GA_X_sel.csv"
        y_matrix_file = "../atividades.txt"

        df = pd.read_csv(os.path.join(directory, X_matrix_file),
                         sep=';', index_col=0)
        X = df.to_numpy()
        y = pd.read_csv(os.path.join(directory, y_matrix_file),
                        sep=';', header=None).values
        cv = CrossValidation(X, y)
        return cv



import logging
from typing import List

import pandas as pd
from Constants import DEBUG_MODE
from MainHandler import Handler
from ResultsHandler import ResultsHandler, ResultWindowTexts
from qsarmodelingpy.Interfaces import ConfigExtValInterface
from qsarmodelingpy.validate_yr_lno import ValidateYRLNOResult
import Plots
from runCalculations import RunCalculations
import os
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class ValidationHandler(Handler):
    def __init__(self, builder, handler: Handler):
        super().__init__(builder)
        self.builder = builder
        self.handler = handler
        self.builder.get_object('config_cv_window').connect(
            'delete-event', lambda w, e: w.hide() or True)
        self.builder.get_object('config_yrlno_window').connect(
            'delete-event', lambda w, e: w.hide() or True)
        self.builder.get_object('config_extval_window').connect(
            'delete-event', lambda w, e: w.hide() or True)

        self.results_handler: ResultsHandler = self.handler.get_handler(ResultsHandler)  # type: ignore
        # self.ext_val_config: ConfigExtValInterface

    def on_cv_run_button_clicked(self, _) -> None:
        if not self.handler.get_X_matrix() or not self.handler.get_y_vector():
            logging.error("Load matrix/vector first.")
            return
        auto = self.builder.get_object("cv_autonLV").get_active()
        nLV = None if auto else self.builder.get_object(
            "cross_validation_nLV").get_value()
        filename = self.builder.get_object("cv_output").get_text()

        logging.debug("Calling RunCalculations.")
        cv = RunCalculations.runCrossValidation(self.handler.get_X_matrix(), self.handler.get_y_vector(), filename, nLV)
        plotter = Plots.CrossValidationPlots()
        params = cv.returnParameters()
        stats = self.__cv_prepare_stats(params)
        texts = ResultWindowTexts(toptext="Your Cross Validation is done.",
                                  title="Cross Validation Results",
                                  statistics=stats)
        self.results_handler.show(cv, plotter, texts)
    
    def __cv_prepare_stats(self, params: pd.DataFrame) -> str:
        stats: List[str] = []
        for i, d in params.iterrows():
            stats.append(f"{i} = {d[0]}")
        return "\n".join(stats)

    def on_yrlno_run_button_clicked(self, _) -> None:
        if not self.handler.get_X_matrix() or not self.handler.get_y_vector():
            logging.error("Please open matrix and vector first.")
            return

        logging.debug("Calling RunCalculations.")
        result: ValidateYRLNOResult = RunCalculations.run_yrlno(
            X_path=self.handler.get_X_matrix(),
            y_path=self.handler.get_y_vector(),
            yr_cut=self.builder.get_object("yrlno_yrand").get_value(),
            lno_cut=self.builder.get_object("yrlno_lno").get_value(),
            return_object=True
        )
        yr_result = result['yr_result']
        yr = yr_result['yr']
        logging.debug("RunCalculations done.")
        logging.debug("Preparing to plot.")
        plotter = Plots.YRandomizationPlots()

        passed_str = "YRandomization passed with score {}." if yr_result["passed"] else "YRandomization failed with score {}.\nEither adjust your desired score or check your data."
        stats = passed_str.format(yr_result["score"])

        texts = ResultWindowTexts(toptext="Your Y-Randomization is done.",
                                    title="Y-Randomization Results",
                                    statistics=stats)
        
        self.results_handler.show(yr, plotter, texts)

    def on_extval_run_button_clicked(self, _) -> None:
        if not self.handler.files_ok():
            logging.error("Please, open the files in File > Open...")
            raise FileNotFoundError("Please, open the files in File > Open...")

        # Choose defaults to output files
        X_name = os.path.splitext(os.path.basename(self.handler.get_X_matrix()))[0]
        basename = os.path.join(os.path.dirname(self.handler.get_X_matrix()), f'{X_name}')
        default_outputs = {
            'output_extval': f"{basename}_extval.csv",
            'output_cv': f"{basename}_cv_extval.csv",
            'output_X_train': f"{basename}_X_train_extval.csv",
            'output_y_train': f"{basename}_y_train_extval.csv",
            'output_X_test': f"{basename}_X_test_extval.csv",
            'output_y_test': f"{basename}_y_test_extval.csv",
        }

        self.ext_val_config: ConfigExtValInterface = {
            'XMatrix': self.handler.get_X_matrix(),
            'yvector': self.handler.get_y_vector(),
            'test_set': self.builder.get_object('extval_test_set').get_text(),
            'latent_vars_model': self.builder.get_object('extval_vars_model').get_value() or None,
            'extval_type': self.builder.get_object('extval_type').get_active_id(),
            'output_extval': self.builder.get_object('extval_output_extval').get_text() or default_outputs["output_extval"],
            'output_cv': self.builder.get_object('extval_output_cv').get_text() or default_outputs["output_cv"],
            'output_X_train': self.builder.get_object('extval_output_X_train').get_text() or default_outputs["output_X_train"],
            'output_y_train': self.builder.get_object('extval_output_y_train').get_text() or default_outputs["output_y_train"],
            'output_X_test': self.builder.get_object('extval_output_X_test').get_text() or default_outputs["output_X_test"],
            'output_y_test': self.builder.get_object('extval_output_y_test').get_text() or default_outputs["output_y_test"],
            'autoscale': self.builder.get_object('extval_autoscale').get_active(),
            'lj_transform': self.builder.get_object('extval_ljtransform').get_active(),
        }

        RunCalculations.runExternalValidation(self.ext_val_config)
        logging.info("Done.")



import sys
import logger, logging
from os import path
from ValidationHandler import ValidationHandler
from FilterHandler import FilterHandler
from OPSHandler import OPSHandler
from GAHandler import GAHandler
from MainHandler import Handler
from HandlerFinder import HandlerFinder
from ResultsHandler import ResultsHandler
from Utils import cleanup_temporary_directory, __DIR__
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


def add_all_from_file(files: list, builder: Gtk.Builder) -> None:
    for f in files:
        builder.add_from_file(path.abspath(
            path.join(__DIR__, "Views", f)))


def main():
    logger.init()

    builder: Gtk.Builder = Gtk.Builder()
    add_all_from_file([
        "main.glade",
        "about.glade",
        "ga.glade",
        "ops.glade",
        "varcut.glade",
        "corrcut.glade",
        "autocorrcut.glade",
        "cross_validation.glade",
        "yrlno.glade",
        "external_validation.glade",
        "results.glade",
    ], builder)


    handler = Handler(builder)

    # Register handlers
    handler.register_handler(GAHandler(builder, handler))
    handler.register_handler(OPSHandler(builder, handler))
    handler.register_handler(ResultsHandler(builder))
    handler.register_handler(ValidationHandler(builder, handler))
    handler.register_handler(FilterHandler(builder, handler))

    handlers = [handler] + handler.get_handlers()

    # Connect signals and launch the GUI
    builder.connect_signals(HandlerFinder(handlers))
    window = builder.get_object('main_window')
    window.show_all()
    Gtk.main()

    # Cleanup temporary directory when the GUI is closed
    cleanup_temporary_directory()

if __name__ == '__main__':
    main()


import os
import sys
from pathlib import Path
import logging
import tempfile
import shutil
import subprocess
import time
import platform
import configparser
from typing import Optional, Union
__DIR__ = Path(getattr(sys, '_MEIPASS', Path(__file__).parent.resolve()))

TMP_DIRECTORY: Optional[str] = None
LOG_FILE: Optional[str]  = None


def __initialize_temporary_file() -> str:
    global LOG_FILE
    LOG_FILE = os.path.join(get_tmp(), "qsarmodeling.log")
    open(LOG_FILE, "w").close()
    return LOG_FILE


def get_log_file() -> str:
    if LOG_FILE is None:
        return __initialize_temporary_file()
    else:
        return LOG_FILE


def get_tmp() -> str:
    global TMP_DIRECTORY
    if TMP_DIRECTORY is None:
        TMP_DIRECTORY = tempfile.mkdtemp()
    return TMP_DIRECTORY


def cleanup_temporary_directory() -> None:
    global TMP_DIRECTORY
    if TMP_DIRECTORY is not None:
        shutil.rmtree(TMP_DIRECTORY)
        TMP_DIRECTORY = None


def open_external(filepath: str) -> None:
    """Open an external file with the default software.
    Args:
        filepath (str): The file path to open
    """
    if platform.system() == 'Darwin':       # macOS
        subprocess.call(('open', filepath))
    elif platform.system() == 'Windows':    # Windows
        os.startfile(filepath) # type: ignore
    else:                                   # linux variants
        subprocess.call(('xdg-open', filepath))


def get_config_filename(filename=None) -> Union[Path, None]:
    """Look for the configuration file and return the path if found.

    Args:
        filename (str | Path | None, optional): the filename to use as default (if this is given and the file exists, it'll be returned). Defaults to None.

    Returns:
        Path: The Path to the configuration file, or None if not found.
    """
    if filename is not None and Path(filename).is_file():
        return Path(filename)

    LOOKUP = [Path("./qsarmodelingpy.conf"),
              Path("~/qsarmodelingpy.conf").expanduser(),
              Path("~/.qsarmodelingpy.conf").expanduser(),
              Path("./qsarmodeling.conf"),
              Path("~/qsarmodeling.conf").expanduser(),
              Path("~/.qsarmodeling.conf").expanduser(),
              __DIR__ / "qsarmodelingpy.conf"
              ]
    for path in LOOKUP:
        if path.resolve().is_file():
            logging.info(f"Loaded {path.resolve()} as the configuration file.")
            return path
    return None


def read_config(section: str = None, key: str = None, filename: Union[Path, str, None] = None) -> Union[str, dict]:
    """Reads a configuration file.

    Args:
        section (str): The section of the configuration file. If None, the whole file is returned.
        key (str): The key of the configuration file. If None, the whole section is returned.

    Returns:
        str: The value of the configuration file. If the file or section/key doesn't exist, an empty dict is returned.
    """
    filename = get_config_filename(filename)
    if filename is None:
        return {}
    parser = configparser.ConfigParser()
    parser.read(filename)
    if section is None:
        if key is not None:
            for sec in parser.sections():
                if key in parser[sec]:
                    return parser[sec][key]
            return {}
        return {section: dict(parser[section]) for section in parser.sections()}
    elif section not in parser.sections():
        return {}
    elif key is None:
        return dict(parser[section])
    else:
        return parser[section][key] if key in parser[section] else {}


# GUI Handlers
def set_output_matrix_as_input(self, config) -> None:
    """Sets the output matrix as input in the GUI.

    It's particularly useful at the end of a calculation, when you want that the result is shown in the GUI.
    """
    if os.path.isfile(config['output_matrix']):
        self.handler.set_X_matrix(config['output_matrix'])
        self.draw_matrices('matrix')
    if self.running_process is not None:
        self.running_process.terminate()
    self.running_process = None


def get_current_time_as_string() -> str:
    """Returns the current time as a string.

    Returns:
        str: The current time as a string.
    """
    return time.strftime("%Y%m%d_%H%M%S", time.localtime())


from typing import Callable, Dict
import numpy as np
import logging, logger
from qsarmodelingpy.external_validation import ExternalValidation
import matplotlib.pyplot as plt  # TODO: add do dependecies
logger.silence_matplotlib_logger()
from abc import ABC, abstractmethod
from qsarmodelingpy.cross_validation_class import CrossValidation
from qsarmodelingpy.yrandomization import YRandomization
import Utils



class Plots(ABC):

    def __init__(self) -> None:
        matplolib_config = Utils.read_config("matplotlib")
        plt.rcParams.update(matplolib_config)

    @abstractmethod
    def get_methods(self) -> Dict[str, Callable]:
        pass


class CrossValidationPlots(Plots):

    def __init__(self) -> None:
        super().__init__()

    def Q2_R2_evolution(self, cv: CrossValidation):
        xaxis = range(1, cv.nLVMax + 1)
        y1 = cv.R2()
        y2 = cv.Q2()
        plt.plot(xaxis, y1, label="R²", marker='o')
        plt.plot(xaxis, y2, label="Q²", marker='o')
        plt.legend()
        plt.title("Cross-Validation error - PLS")
        plt.xlabel("nLV")
        plt.ylabel(r"$R^2 / Q^2$")
        plt.show()

    def scatter_Q2_R2(self):
        raise NotImplementedError()

    def y_versus_ŷ_CV(self, cv: CrossValidation):
        nLV = int(cv.returnParameters().loc["nLV"][0])
        xaxis = np.array(cv.y)
        yaxis = list(cv.ycv[:, nLV-1])
        plt.plot(xaxis, xaxis, color="red", linewidth=1, label="y = x line")
        plt.plot(xaxis, yaxis, linestyle="None", marker="o", label="Data")
        plt.title("Cross-Validation prediction")
        plt.xlabel("Experimental activity")
        plt.ylabel("Predicted activity")
        plt.show()

    def y_versus_ŷ_cal(self, cv: CrossValidation):
        nLV = int(cv.returnParameters().loc["nLV"][0])
        xaxis = np.array(cv.y)
        yaxis = list(cv.ycal[:, nLV-1])
        plt.plot(xaxis, xaxis, color="red", linewidth=1, label="y = x line")
        plt.plot(xaxis, yaxis, linestyle="None", marker="o", label="Data")
        plt.title("Calibration prediction")
        plt.xlabel("Experimental activity")
        plt.ylabel("Predicted activity")
        plt.show()

    def get_methods(self) -> Dict[str, Callable]:
        return {
            "Q² and R² × Latent Variables (cross-validation)": self.Q2_R2_evolution,
            "Exprimental × Predicted (calibration)": self.y_versus_ŷ_cal,
            "Exprimental × Predicted (cross-validation)": self.y_versus_ŷ_CV,
        }

class YRandomizationPlots(Plots):
    def __init__(self) -> None:
        super().__init__()

    def Q2_versus_R2(self, yr: YRandomization):
        R2_rand = yr.R2[:-1]
        Q2_rand = yr.Q2[:-1]
        R2_best = yr.R2[-1]
        Q2_best = yr.Q2[-1]
        plt.plot(R2_rand, Q2_rand, linestyle="None", marker='o', label="Randomized")
        plt.plot(R2_best, Q2_best, linestyle="None", marker='o', label="Best model")
        plt.title("y-randomization")
        plt.xlabel("R²")
        plt.ylabel("Q²")
        plt.show()

    def get_methods(self) -> Dict[str, Callable]:
        return {
            "Q² and R² × Latent Variables (y-randomization)": self.Q2_versus_R2,
        }


if __name__ == "__main__":
    import pandas as pd
    import os
    import coloredlogs
    import logging
    logging_level = logging.DEBUG
    coloredlogs.install(
        fmt="%(filename)s:%(lineno)s %(funcName)s() %(levelname)s  %(message)s", level=logging_level)
    coloredlogs.DEFAULT_FIELD_STYLES = {'filename': {'color': 'blue'}, 'lineno': {
        'color': 'blue'}, 'funcName': {'color': 'magenta'}, 'levelname': {'bold': True, 'color': 'black'}}

    directory = "/home/helitonmrf/Documents/QSAR/cancer_prostata/resultados"
    X_matrix_file = "filtered_10_GA_X_sel.csv"
    y_matrix_file = "../atividades.txt"

    df = pd.read_csv(os.path.join(directory, X_matrix_file),
                     sep=';', index_col=0)
    X = df.to_numpy()
    y = pd.read_csv(os.path.join(directory, y_matrix_file),
                    sep=';', header=None).values
    cv = CrossValidation(X, y)
    methods = CrossValidationPlots().get_methods()
    # logging.debug(f"{methods = }")

    # for method in methods:
    #     methods[method](cv)
    #     break

    yr = YRandomization(X,y,cv.nLVMax)
    yrp = YRandomizationPlots()
    yrp.Q2_versus_R2(yr)



import sys
import logging
import coloredlogs
from Constants import DEBUG_MODE
from Utils import get_log_file


def init():
    logging_level = logging.DEBUG if DEBUG_MODE else logging.INFO
    if not DEBUG_MODE:
        # redirect stderr and stdout to a temporary file
        LOG_FILE = get_log_file()
        print(f"Initialized log in {LOG_FILE}")
        sys.stdout = open(LOG_FILE, 'a')
        sys.stderr = sys.stdout

    coloredlogs.install(fmt="%(filename)s:%(lineno)s %(funcName)s() %(levelname)s  %(message)s", level=logging_level)
    coloredlogs.DEFAULT_FIELD_STYLES = {'filename': {'color': 'blue'}, 'lineno': {'color': 'blue'}, 'funcName': {'color': 'magenta'}, 'levelname': {'bold': True, 'color': 'black'}}


def silence_matplotlib_logger():
    logging.getLogger('matplotlib').setLevel(logging.WARNING)


import setuptools

long_description = """
# QSARModelingPy

QSARModelingPy is an open-source computational package to generate and validate QSAR models.

**What you _can_ do with QSARModelingPy**

-   Select variables through either OPS or Genetic Algorithm
-   Dimensionality reduction:
    -   Correlation cut
    -   Variance cut
    -   Autocorrelation cut
-   Validate your models:
    -   Cross Validation
    -   y-randomization / Leave-N-out
    -   External Validation

## Usage
After installing this package with:
```
$ pip install qsarmodelingpy-gui
```
Just launch the application:
```
$ qsarmodelingpy
```
or:
```
$ python -m qsarmodelingpy-gui
```
"""

setuptools.setup(
    name="qsarmodelingpy-gui",
    version="0.2.2",
    author="Reis Filho, H. M.; Martins, J. P. A",
    author_email="helitonmrf@ufmg.br,jpam@qui.ufmg.br",
    description="A software for building and validating QSAR models",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hellmrf/QSARModelingPyInterfaces",
    packages=["qsarmodelingpy-gui"],
    package_dir={"qsarmodelingpy-gui": "GUI"},
    package_data={"qsarmodelingpy-gui": ["Views/*.glade"]},
    install_requires=[
        'qsarmodelingpy',
        'pygobject==3.30.5',
        'typing_extensions',
        'coloredlogs',
        'matplotlib',
        'tornado',
        'PyQt5',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'qsarmodelingpy = qsarmodelingpy_gui.main:main',
        ],
    },
    python_requires='>=3.6',
)



import os
import gi
import time
import logging
from multiprocessing import Process
from threading import Thread
from Interfaces import ConfigGAInterface
from MainHandler import Handler
from runCalculations import RunCalculations
from Utils import set_output_matrix_as_input, get_current_time_as_string
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class GAHandler(Handler):
    def __init__(self, builder, handler: Handler):
        super().__init__(builder)
        self.builder = builder
        self.handler = handler
        self.config_GA_window = self.builder.get_object('config_GA_window')
        self.config_GA_window.connect(
            'delete-event', lambda w, e: w.hide() or True)
        self.ga_config: ConfigGAInterface
        self.running_process = None

    def _is_running(self) -> bool:
        if self.running_process is None:
            return False
        if self.running_process.is_alive():
            return True
        else:
            self.running_process.terminate()
            return False

    def GA_confirm_close(self, dialog, response) -> None:
        dialog.hide()
        if response == Gtk.ResponseType.OK and self._is_running():
            self.on_GA_cancel_button_clicked(None, force=True)

    def on_GA_cancel_button_clicked(self, _, force: bool = False) -> None:
        """ Handle GA cancel button """
        if self._is_running():
            logging.debug(f"running_process = {self.running_process}")
            if not force:
                self.builder.get_object('GA_will_kill_all_window').show()
                return
            else:
                logging.warning("Terminating GA Calculation.")
                self.running_process.terminate()
                self.running_process = None
        self.config_GA_window.hide()

    def _keep_updating_progress_bar(self) -> None:
        pbar = self.builder.get_object("GA_progress_bar")
        pbar.show()
        while self._is_running():
            time.sleep(0.1)
            pbar.pulse()
        else:
            pbar.set_fraction(0)
            # If everything is ok, current matrix will be the filtered one.
            set_output_matrix_as_input(self, self.ga_config)

    def on_GA_run_button_clicked(self, _) -> None:
        if self._is_running():
            logging.warning(
                f"Already running with PID = {self.running_process.pid}")
            return

        if self.handler.files_ok():
            self.ga_config = {
                'XMatrix': self.handler.get_X_matrix(),
                'yvector': self.handler.get_y_vector(),
                'output_matrix': self.builder.get_object('GA_output_matrix').get_text(),
                'output_cv': self.builder.get_object('GA_output_cv').get_text(),
                'output_q2': self.builder.get_object('GA_output_q2').get_text(),
                'output_selected': self.builder.get_object('GA_output_selected_variables').get_text(),
                'output_PLS_model': self.builder.get_object('GA_output_model').get_text(),
                'varcut': self.builder.get_object('ga_varcut').get_value(),
                'corrcut': self.builder.get_object('ga_corrcut').get_value(),
                'autoscale': self.builder.get_object('ga_autoscale').get_active(),
                'lj_transform': self.builder.get_object('ga_ljtransform').get_active(),
                'autocorrcut': self.builder.get_object('ga_autocorrcut').get_value(),
                'max_latent_model': self.builder.get_object('ga_max_latent_model').get_value(),
                'min_vars_model': self.builder.get_object('ga_min_vars_model').get_value(),
                'max_vars_model': self.builder.get_object('ga_max_vars_model').get_value(),
                'population_size': self.builder.get_object('ga_population_size').get_value(),
                'migration_rate': self.builder.get_object('ga_migration_rate').get_value(),
                'crossover_rate': self.builder.get_object('ga_crossover_rate').get_value(),
                'mutation_rate': self.builder.get_object('ga_mutation_rate').get_value(),
                'generations': self.builder.get_object('ga_generations').get_value(),
                'yrand': self.builder.get_object('ga_yrand').get_value(),
                'lno': self.builder.get_object('ga_lno').get_value()
            }

            date_time = get_current_time_as_string()
            if not self.ga_config['output_matrix']:
                self.ga_config['output_matrix'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()),
                                                               "GA_output_matrix_{}.csv".format(date_time))
            if not self.ga_config['output_cv']:
                self.ga_config['output_cv'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()),
                                                           "GA_output_CV_{}.csv".format(date_time))
            if not self.ga_config['output_q2']:
                self.ga_config['output_q2'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()),
                                                           "GA_output_Q2_{}.csv".format(date_time))
            if not self.ga_config['output_selected']:
                self.ga_config['output_selected'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()),
                                                                 "GA_output_selected_{}.csv".format(date_time))
            if not self.ga_config['output_PLS_model']:
                self.ga_config['output_PLS_model'] = os.path.join(os.path.dirname(self.handler.get_X_matrix()),
                                                                 "GA_output_model_{}.csv".format(date_time))

            logging.debug("Ok, I'll call RunCalculations.runGA().")
            self.running_process = Process(target=GAHandler.call_runner, args=(self.ga_config,))
            self.running_process.start()
            pbthread = Thread(target=self._keep_updating_progress_bar)
            pbthread.start()
            logging.debug(f"Started PID = {self.running_process.pid}")

        else:
            logging.error("Please, open the files in File > Open...")

    @staticmethod
    def call_runner(config: ConfigGAInterface) -> None:
        logging.debug("Calling GA runner")
        RunCalculations.runGA(config)
        logging.debug("Calculation done.")


import logging


class HandlerFinder(object):
    """Searches for handler implementations across multiple objects.
    """

    # See <http://stackoverflow.com/questions/4637792> for why this is
    # necessary.

    def __init__(self, backing_objects):
        self.backing_objects = backing_objects

    def __getattr__(self, name):
        for o in self.backing_objects:
            if hasattr(o, name):
                return getattr(o, name)
        else:
            logging.error("{} not found on any of {}".format(
                name, self.backing_objects))
            raise AttributeError("%r not found on any of %r"
                                 % (name, self.backing_objects))



import numpy as np
import pandas
import os
import json
import logging
from qsarmodelingpy.runGa import run as runGA
from qsarmodelingpy.runOPS import run as runOPS
from qsarmodelingpy.runExtVal import run as runExtVal
from qsarmodelingpy.filter import variance_cut, correlation_cut, autocorrelation_cut
from qsarmodelingpy.cross_validation_class import CrossValidation
from qsarmodelingpy.validate_yr_lno import validate, run_leavenout, run_yrandomization, ValidateYRLNOResult, YRResult, LNOResult
from qsarmodelingpy.Interfaces import ConfigGAInterface, ConfigOPSInterface, ExtValResult, ConfigExtValInterface
from qsarmodelingpy.Utils import load_matrix
from typing import Union


class RunCalculations:

    @staticmethod
    def runGA(config: ConfigGAInterface) -> bool:
        return runGA(config)

    @staticmethod
    def runOPS(config: ConfigOPSInterface) -> bool:
        return runOPS(config)

    @staticmethod
    def runVarCut(filename: str, value: float, save: bool = True, output: str = "") -> str:
        df = load_matrix(filename)
        indVar = variance_cut(df.values, value)
        dfCut = df.loc[:, df.columns[indVar]]
        if save:
            if not output:
                name = os.path.split(filename)[-1][:-4]
                output = os.path.join(os.path.dirname(filename),
                                      "{}_filtered_var_{:.2f}.csv".format(name, value))
            dfCut.to_csv(output)
            return output
        else:
            """ In the future, the user will be able to cut the matrix without 
             saving it, leaving it temporarily available within the program to
             perform another calculation in the sequence. """
            raise NotImplementedError(
                "save=False is reserved for future usage. Please, use save=True.")

    @staticmethod
    def runCorrelationFilter(auto: bool, X_path: str, y_path: str, value: float, save: bool = True, output: str = "") -> str:
        dfX = load_matrix(X_path)
        dfy = pandas.read_csv(y_path, header=None)
        indVar = autocorrelation_cut(dfX.values, dfy, value) if auto else correlation_cut(
            dfX.values, dfy.values, value)
        dfCut = dfX.loc[:, dfX.columns[indVar]]
        if save:
            if not output:
                name = os.path.split(X_path)[-1][:-4]
                output = os.path.join(os.path.dirname(X_path),
                                      "{}_filtered_{}_{:.2f}.csv".format(name, "autocorr" if auto else "corr", value))
            dfCut.to_csv(output)
            return output
        else:
            """ In the future, the user will be able to cut the matrix without 
             saving it, leaving it temporarily available within the program to
             perform another calculation in the sequence. """
            raise NotImplementedError(
                "save=False is reserved for future usage. Please, use save=True.")

    @staticmethod
    def runCorrCut(X_path: str, y_path: str, corrcut: float, save: bool = True, output: str = "") -> str:
        return RunCalculations.runCorrelationFilter(False, X_path, y_path, corrcut, save, output)

    @staticmethod
    def runAutoCorrCut(X_path: str, y_path: str, autocorrcut: float, save: bool = True, output: str = "") -> str:
        return RunCalculations.runCorrelationFilter(True, X_path, y_path, autocorrcut, save, output)

    @staticmethod
    def runCrossValidation(X_path: str, y_path: str, filename: str = "", nLV=None, skipsaving=False) -> CrossValidation:
        if filename == "":
            X_name = os.path.splitext(os.path.basename(X_path))[0]
            filename = os.path.join(os.path.dirname(X_path),
                                    f'{X_name}_CV_output.csv')
        dfX = load_matrix(X_path).values
        dfy = pandas.read_csv(y_path, header=None).values
        logging.debug(dfX.shape)
        logging.debug(dfy.shape)
        logging.debug(dfy)
        cv = CrossValidation(dfX, dfy)
        if not skipsaving:
            cv.saveParameters(filename)
        return cv

    @staticmethod
    def run_yrlno(X_path: str,
                  y_path: str,
                  yr_cut: float = 0.3,
                  lno_cut: float = 0.1,
                  return_object=False
                  ) -> Union[bool, ValidateYRLNOResult]:
        dfX = load_matrix(X_path)
        dfy = pandas.read_csv(y_path, header=None).to_numpy()
        yrresult = run_yrandomization(dfX, dfy, yr_cut)
        lnoresult = run_leavenout(dfX, dfy, lno_cut)

        passed = yrresult["passed"] and lnoresult["passed"]
        
        result = ValidateYRLNOResult({"passed": passed, "yr_result": yrresult, "lno_result": lnoresult})
        if return_object:
            return result
        else:
            return passed



    @staticmethod
    def runExternalValidation(config: ConfigExtValInterface) -> ExtValResult:
        return runExtVal(config)
